! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: hkstrip_mod
!
! Authors:  Dean Hinshaw - University of Leicester
!          
! Description: Remove un-needed columns from HK files.
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module hkstrip_mod

  use dal
  use oal
  use param  
  
  implicit none 
  
  private 
  
  public hkstrip

contains 

  !---------------------------------------------------------------------
  subroutine hkstrip
  !---------------------------------------------------------------------
    
    implicit none

    character(LEN=1024) :: extension, colname, filename
    character(LEN=*), parameter :: hkparam = "paramnames", in_file = "set", &
         out_file = "outset", extname = "table", useodf="useodf"

    integer :: nrows, icol, irow

    type(DataSetT) :: in_set, out_set
    type(TableT)   :: in_tab, out_tab
    type(ColumnT)  :: in_col, out_col

    real(kind=double), dimension(:), pointer :: realTime
    
    if( booleanParameter(useodf) )then
       call OAL_expandFileName(stringParameter(in_file), filename)
       if( trim(filename) == "" )then
          write(errstr, *) "The file name ", trim(filename), &
               " is not a valid ODF file specification."
          call error("BadODFname")
       end if
    else
       filename = stringParameter(in_file)
    end if

    ! Prep the outgoing fits files.

    extension = stringParameter(extname)

    in_set = dataSet(filename, read)
    in_tab = table(in_set, extension)
    nrows = numberOfRows(in_tab)

    out_set = dataSet(stringParameter(out_file), Create)
    out_tab = addTable(out_set, extension, nrows)

    call addAttributes(attributable(out_set),attributable(in_set))
    call addAttributes(attributable(out_tab),attributable(in_tab))

    ! Now copy to the new file the selected columns

    do icol = 0, parameterCount(hkparam)-1
       colname = stringParameter(hkparam,icol)
       in_col = column(in_tab, colname, read)

       if( booleanParameter(useodf) .and. trim(colname)=="TIME" .and. &
            columnDataType(in_col)==String )then
          out_col = addColumn(out_tab, "TIME", Real64)
          call setAttribute(out_tab, "MJDREF", 50814D+0, "1998-01-01T00:00:00 (TT) expressed in MJD(TT)")
          call setAttribute(out_tab, "TIMEUNIT", "s", "All times in s unless specified otherwise")
          call setAttribute(out_tab, "TIMESYS", "TT", "XMM time will be TT (Terrestial Time)")
          call setAttribute(out_tab, "TIMEZERO", 0, "Clock correction")
          call setAttribute(out_tab, "CLOCKAPP", "T", "Clock correction applied")
          call setAttribute(out_tab, "TIMEREF", "LOCAL", "Reference location of photon arrival times")
          call setAttribute(out_tab, "TASSIGN", "SATELLITE", "Location of time assignment")

          realTime => real64Data(out_col)
          do irow=0, nrows-1
             realTime(irow) = OAL_ertToTimeTag(stringCell(in_col,irow))
          end do
       else
          call copyColumn(out_tab, in_col)
          out_col = column(out_tab, colname, modify)
          call addAttributes(attributable(out_col),attributable(in_col))
       endif

       call release(out_col)
       call release(in_col)
    end do

    call release(in_tab)
    call release(out_tab)
    call release(in_set)
    call release(out_set)

  end subroutine hkstrip
  
end module hkstrip_mod
